#!/bin/bash
. /etc/os-release
OS="$ID $VERSION_ID"

echo "Starting autoupdate"
echo "Detected OS: $OS"

if [ -f /usr/sbin/asterisk ]
then
  #Check current calls
  echo "Checking call count"
  CNT=`/usr/sbin/asterisk -rx "core show calls" | grep "activ" | cut -d " " -f 1`
  if [ "$CNT" != "0" ]
  then
    echo "Can't update: there are $CNT calls currently"
    exit 1
  fi 
fi

if [ -f /etc/cpbx/cpbx-version ]
then
  #if we are on centos, we must check if xorcom-centos-release package is installed
  if [ "$ID" = "centos" ]
  then
    echo "Checking dependencies"
    CNT=`rpm -qa | grep xorcom-centos-release | wc -l`
    if [ "$CNT" == "0" ]
    then
      echo "Can't update: package xorcom-centos-release missing"
      exit 1
    fi
  fi
fi


if [ -f /usr/bin/mysql ]
then
  #check Mysql running state
  echo "Checking Mysql state"
  mysql -u root -e "SHOW databases" > /dev/null
  if [ "$?" != "0" ]
  then 
    echo "Can't update: Mysql is not running, probably broken. please check and call support team"
    exit 1
  fi

  #check mysql data integrity
  echo "Checking Mysql integrity"
  mysqlcheck -u root --all-databases --check > /dev/null
  if [ "$?" != "0" ]
  then 
    echo "Can't update: Mysql data corrupted. Please check and call support team"
    exit 1
  fi
fi


# check size available
# we must have at least 1GB of available space, and at least as much space available as mysql data dir + 1G
echo "Checking disk usage"
MYSQLSIZE=`du -b -d 0 /var/lib/mysql | cut -f1`
NEEDDISKFREE=$(($MYSQLSIZE + 1073741824))
DISKFREE=`df / --output=avail | tail -n 1`
DISKFREE=$(($DISKFREE * 1024))
if [ $DISKFREE -lt $NEEDDISKFREE ]
then
  echo "Can't update: not enough disk space for automatic update. Please do manual update !"
  exit 1
else 
  echo "I need $DISKFREE bytes of available disk, i have $DISKFREE, OK"
fi


#seems ok, update !
echo "Adding repositories"
if [ "$OS" = "centos 7" ]
then
  cat > /etc/yum.repos.d/ipconnect-direct.repo <<-EOF
[ipconnect-direct]
name=IPConnect software repository
baseurl=http://repository.ipconnect.fr/Centos/7
enabled=1
gpgcheck=0

[ipconnect-direct-beta]
name=IPConnect software repository
baseurl=http://repository.ipconnect.fr/Centos/7-beta
enabled=0
gpgcheck=0
EOF
elif [ "$OS" = "debian 8" ]
then
  echo "deb http://repository.ipconnect.fr/Debian/ jessie main" > /etc/apt/sources.list.d/ipconnect-direct.list
  apt-key adv --keyserver keyserver.ubuntu.com --recv-keys D072693A > /dev/null
elif [ "$OS" = "debian 11" ] 
then
  echo "deb http://repository.ipconnect.fr/Debian/ bullseye main" > /etc/apt/sources.list.d/ipconnect-direct.list
  apt-key adv --keyserver keyserver.ubuntu.com --recv-keys D072693A > /dev/null
elif [ "$OS" = "debian 12" ] 
then
  echo "deb http://repository.ipconnect.fr/Debian/ bookworm main" > /etc/apt/sources.list.d/ipconnect-direct.list
  apt-key adv --keyserver keyserver.ubuntu.com --recv-keys D072693A > /dev/null
fi

echo "Updating..."
xlic auth
if [ "$OS" = "centos 7" ]
then
  yum clean expire-cache
  yes | yum update
else
  apt update
  env DEBIAN_FRONTEND=noninteractive APT_LISTCHANGES_FRONTEND=mail apt-get dist-upgrade -y -o Dpkg::Options::='--force-confdef' -o Dpkg::Options::='--force-confold'
fi
exit $?