#!/bin/bash
# this script try to guess if FS/disk is corrupted. 
# the detection is more or like try and guess...
. /etc/os-release
OS="$ID $VERSION_ID"
echo "Detected OS: $OS"
DEBUG=1
error=0

checkSmartctl() {
  smartoutput=$(smartctl -v 1,raw48:54 -v 7,raw48:54 -v 195,raw48:54 -a $1)

  # SSD Wear Level
  #var=`smartctl -a $1 | grep Wear_Leveling | awk '{print $4}' | sed 's/^0\|^00//'`
  var=`echo "$smartoutput" | egrep -i "177 Wear_Leveling|231 SSD_Life_Left|^173 Un|233 Media_Wearout_" | awk '{print $4}' | sed 's/^0\|^00//' | head -n 1`
  if [[ ${var#0} -lt 20 ]] && [[ ${var#0} -gt 0 ]]
  then
    echo -e "\e[0;41m$1 is at ${var#0}% SSD wear\e[0m"
    error=$(($error + 1))
  fi

  # Reallocated sectors on SATA drives
  #var=`smartctl -a $1 | grep Reallocated_Sector | awk '{print $10}' `
  var=`echo "$smartoutput" | grep Reallocated_Sector | awk '{print $10}' `
  if [[ $var -gt 0 ]]
  then
    echo -e "\e[0;41m$1 has $var Sector Errors\e[0m"
    error=$(($error + 1))
  fi

  # Early Warning Offline_Uncorrectable
  #var=`smartctl -a $1 | grep Offline_Uncorrectable | awk '{print $10}' `
  var=`echo "$smartoutput" | grep Offline_Uncorrectable | awk '{print $10}' `
  if [[ $var -gt 0 ]]
  then
    echo -e "\e[0;41m$1 has $var Offline Uncorrectable Errors\e[0m"
    error=$(($error + 1))
  fi

  # Early Warning Raw_Read_Error_Rate
  var=`echo "$smartoutput" | egrep -i "1 Raw_Read_Error_Rate" | awk '{print $10}' | sed 's/^0\|^00//'`
  if [[ ${var#0} -gt 10 ]] && [[ ${var#0} -gt 0 ]]
  then
    echo -e "\e[0;41m$1 has a Read Error Rate of ${var#0}\e[0m"
    error=$(($error + 1))
  fi

  # SAS Read errors
  #var=`smartctl -a $1 | egrep "read:" | awk '{print $8}'`
  var=`echo "$smartoutput" | egrep "read:" | awk '{print $8}'`
  if [[ $var -gt 0 ]]
  then
    echo -e "\e[0;41m$1 $var SAS Read Errors\e[0m"
    error=$(($error + 1))
  fi

  # SAS Write errors
  #var=`smartctl -a $1 | egrep "write:" | awk '{print $8}'`
  var=`echo "$smartoutput"  | egrep "write:" | awk '{print $8}'`
  if [[ $var -gt 0 ]]
  then
    echo -e "\e[0;41m$1 $var SAS Write Errors\e[0m"
    error=$(($error + 1))
  fi

  # SAS Verify errors
  #var=`smartctl -a $1 | egrep "verify:" | awk '{print $8}'`
  var=`echo "$smartoutput"  | egrep "verify:" | awk '{print $8}'`
  if [[ $var -gt 0 ]]
  then
    echo -e "\e[0;41m$1 $var SAS Verify Errors\e[0m"
    error=$(($error + 1))
  fi
}



# if smartctl is installed
if [ -x "$(command -v smartctl)" ]
then
  # for each /dev/sdx, 
  lsblk -l -o NAME,TYPE | grep disk | cut -f1 | while read -r disk; do
    checkSmartctl /dev/$disk
  done
fi

# check errors in dmesg about dma transfer 
var=$(dmesg | grep -i "DMA transfer could not be terminated" | wc -l)
error=$(($error + $var))
if [ $var -gt 0 ]
then
  echo -e "\e[30;42mDisk has $var DMA transfer errors !\e[0m"
fi

# check errors in dmesg about mmcblk0
var=$(dmesg | grep -i "mmcblk0: error" | wc -l)
error=$(($error + $var))
if [ $var -gt 0 ]
then
  echo -e "\e[30;42mDisk has $var mmcblk0 errors !\e[0m"
fi

# check errors not corrected about f2fs
var=$(dmesg | grep -i "Cannot recover all fsync data errno" | wc -l)
error=$(($error + $var))
if [ $var -gt 0 ]
then
  echo -e "\e[30;42mF2FS errors founds\e[0m"
fi

# check errors correction about f2fs
# Retiré car détecte des erreurs résolues sur swan... 
#var=$(dmesg | grep -i "f2fs" | grep -i "recovered" | wc -l)
#error=$(($error + $var))
#if [ $var -gt 0 ]
#then
#  echo -e "\e[30;42mF2FS errors founds\e[0m"
#fi

# try writing file
TMPDIR=$(mktemp > /dev/null)
if [ "$?" != "0" ]
then
  error=$(($error + 1))
  echo -e "\e[30;42mFailed to write test file\e[0m"
else 
  rm -f $TMPDIR
fi



echo "Error(s) found:$error"
exit $error;
